package pp.s1184725.boppi;

import static pp.s1184725.boppi.BasicLexer.*;

import pp.iloc.eval.Machine;

/**
 * This class provides 4 basic data types:
 * <ul>
 * <li>character (sized {@link Machine#DEFAULT_CHAR_SIZE})</li>
 * <li>integer (sized {@link Machine#INT_SIZE})</li>
 * <li>boolean (sized {@link Machine#INT_SIZE})</li>
 * <li>void (empty/unit type)</li>
 * </ul>
 */
public enum SimpleType {
	INT(Machine.INT_SIZE), CHAR(Machine.DEFAULT_CHAR_SIZE), BOOL(Machine.INT_SIZE), VOID(0);

	private final int size;

	/**
	 * Gets the size of this data type.
	 * 
	 * @return the size (in bytes) of this type
	 */
	public int getSize() {
		return size;
	}

	private SimpleType(int typeSize) {
		size = typeSize;
	}

	/**
	 * Maps a token lexed by {@link BasicLexer} to a data type.
	 * 
	 * @param token
	 *            the token to parse
	 * @return the data type of the token, {@link SimpleType#VOID} if
	 *         unrecognized
	 */
	public static SimpleType fromToken(int token) {
		switch (token) {
		case INTTYPE:
			return SimpleType.INT;

		case CHARTYPE:
			return SimpleType.CHAR;

		case BOOLTYPE:
			return SimpleType.BOOL;

		default:
			return SimpleType.VOID;
		}
	}
}
