package pp.s1184725.boppi.test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import java.util.List;
import java.util.logging.*;

import org.antlr.v4.runtime.CharStream;
import org.junit.Test;

import pp.s1184725.boppi.ToolChain;

public class CheckerTest {
	private List<LogRecord> log;

	private void checkString(String code) {
		check(ToolChain.getCharStream(code));
	}

	private void checkFile(String file) {
		check(ToolChain.getCharStream(AllTests.directory.resolve(file)));
	}

	private void check(CharStream stream) {
		Logger logger = Logger.getAnonymousLogger();
		log = ToolChain.makeListLog(logger);
		ToolChain.getAnnotations(ToolChain.getParser(ToolChain.getLexer(stream, logger), logger).program(), logger);
	}

	@Test
	public void correctExpressionTest() {
		checkFile("simpleExpression.boppi");
		assertThat(log, is(empty()));
	}

	@Test
	public void wrongExpressionTest() {
		checkString("+true");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("5 || true");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("6 + 'c'");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("4 + print(5, 6)");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("print(print(3, 5))");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctVariableTest() {
		checkFile("simpleVariable.boppi");
		assertThat(log, is(empty()));
	}

	@Test
	public void wrongVariableTest() {
		checkString("var bool name; name := 5");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("undefinedName");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("var undefinedType name; 1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("var bool endsWithDeclaration;");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("var bool var1; var var1 var2; var2 := 'c' ");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctScopeTest() {
		checkFile("simpleScope.boppi");
		assertThat(log, is(empty()));
	}

	@Test
	public void wrongScopeTest() {
		checkString("var bool var1; var bool var1; 1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("var bool var1; var char var1; 1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));

		checkString("{ var int var1; var1 := 4}; var int var2; var1");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getLevel(), is(Level.SEVERE));
	}

	@Test
	public void correctConditionalTest() {
		checkFile("if.boppi");
		assertThat(log, is(empty()));

		checkFile("while.boppi");
		assertThat(log, is(empty()));
	}
}
