package pp.s1184725.boppi.test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import java.util.List;
import java.util.logging.*;

import org.antlr.v4.runtime.CharStream;
import org.antlr.v4.runtime.tree.ParseTree;
import org.junit.Test;

import pp.iloc.model.Program;
import pp.s1184725.boppi.*;

public class GeneratorTest {
	private List<LogRecord> log;
	private String[] out;

	private void compileAndRunString(String code, String... input) {
		compileAndRun(ToolChain.getCharStream(code), String.join("\n", input) + "\n");
	}

	private void compileAndRunFile(String file, String... input) {
		compileAndRun(ToolChain.getCharStream(AllTests.directory.resolve(file)), String.join("\n", input) + "\n");
	}

	private void compileAndRun(CharStream stream, String input) {
		Logger logger = Logger.getAnonymousLogger();
		log = ToolChain.makeListLog(logger);

		ParseTree ast = ToolChain.getParser(ToolChain.getLexer(stream, logger), logger).program();
		Annotations annotations = ToolChain.getAnnotations(ast, logger);
		Program program = ToolChain.getILOC(ast, annotations);
		out = ToolChain.execute(program, logger, input).split("\n");
	}

	@Test
	public void correctExpressionTest() {
		compileAndRunFile("simpleExpression.boppi");
		assertThat(log, is(empty()));
	}

	@Test
	public void wrongExpressionTest() {
		compileAndRunString("1/0");
		assertThat(log, hasSize(1));
		assertThat(log.get(0).getMessage(), containsString("zero"));
	}

	@Test
	public void correctVariableTest() {
		compileAndRunFile("simpleVariable.boppi");
		assertThat(log, is(empty()));
	}

	@Test
	public void basicPrograms() {
		compileAndRunString("print(5*3)");
		assertThat(log, is(empty()));
		assertThat(out, is(arrayContaining("15")));

		compileAndRunString("print({var int x; x := 8; print(x)})");
		assertThat(log, is(empty()));
		assertThat(out, is(arrayContaining("8", "8")));

		compileAndRunString("print('T', 'e', 's', 't', '!')");
		assertThat(log, is(empty()));
		assertThat(out, is(arrayContaining("T", "e", "s", "t", "!")));

		compileAndRunString("var int x; var int y; x := 3*(y := 4); print(x,y)");
		assertThat(log, is(empty()));
		assertThat(out, is(arrayContaining("12", "4")));

		compileAndRunFile("basicProgram1.boppi", "1", "T");
		assertThat(log, is(empty()));
		assertThat(out, is(arrayContaining("T", "T")));

		compileAndRunFile("fibonacciIterative.boppi", "6");
		assertThat(log, is(empty()));
		assertThat(out, is(arrayContaining("13")));
	}
}
